﻿Imports C1.Silverlight.FlexGrid
Imports System.ComponentModel
Imports System.Windows.Media.Imaging

Partial Public Class MainPage
    Inherits UserControl

    Public Sub New()
        InitializeComponent()

        Dim col As New Column()

        col.ColumnName = "FirstName"
        col.Header = "First Name"
        c1FlexGrid1.Columns.Add(col)

        col = New Column()
        col.ColumnName = "LastName"
        col.Header = "Last Name"
        c1FlexGrid1.Columns.Add(col)


        For j As Integer = 0 To 4
            c1FlexGrid1.Rows.Add(New Row())
        Next

        c1FlexGrid1(0, 0) = "John"
        c1FlexGrid1(0, 1) = "Scott"

        c1FlexGrid1(1, 0) = "William"
        c1FlexGrid1(1, 1) = "Trevor"

        c1FlexGrid1(2, 0) = "Michael"
        c1FlexGrid1(2, 1) = "Mayes"


    End Sub

    Private Sub c1FlexGrid1_Click(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles c1FlexGrid1.Click
        Dim ht = c1FlexGrid1.HitTest(e)

        If ht.CellType = C1.Silverlight.FlexGrid.CellType.ColumnHeader Then

            Dim col = c1FlexGrid1.Columns(ht.Column)

            Dim sortDirection = If((TypeOf col.Tag Is ListSortDirection) AndAlso DirectCast(col.Tag, ListSortDirection) = ListSortDirection.Ascending, ListSortDirection.Descending, ListSortDirection.Ascending)

            col.Tag = sortDirection

            For i As Integer = 0 To c1FlexGrid1.Columns.Count - 1
                If Not i = ht.Column Then
                    c1FlexGrid1.Columns(i).Tag = Nothing
                End If
            Next

            SortColumn(col, sortDirection)
        End If

        c1FlexGrid1.CellFactory = New SortCellFactory()
    End Sub

    Private Sub SortColumn(col As Column, sortDirection As ListSortDirection)
        Dim list = New List(Of Row)()

        Dim emptyRows = New List(Of Row)()
        Dim isEmptyRow As Boolean = True

        For Each row In c1FlexGrid1.Rows
            For Each column In c1FlexGrid1.Columns
                If c1FlexGrid1(row.Index, column.Index) Is Nothing OrElse c1FlexGrid1(row.Index, column.Index) = "" Then
                    isEmptyRow = True
                Else
                    isEmptyRow = False
                    Exit For
                End If
            Next

            If isEmptyRow Then
                emptyRows.Add(row)
            Else
                list.Add(row)
            End If
        Next

        list.Sort(New RowComparer(col, sortDirection))

        c1FlexGrid1.Rows.Clear()

        For Each row In list
            c1FlexGrid1.Rows.Add(row)
        Next

        For Each row In emptyRows
            c1FlexGrid1.Rows.Add(row)
        Next
    End Sub

    Private Class RowComparer
        Implements IComparer(Of C1.Silverlight.FlexGrid.Row)

        Private _col As Column
        Private _sortDirection As ListSortDirection

        Public Sub New(col As Column, sortDirection As ListSortDirection)
            _col = col
            _sortDirection = sortDirection
        End Sub

        Public Function Compare(x As C1.Silverlight.FlexGrid.Row, y As C1.Silverlight.FlexGrid.Row) As Integer Implements System.Collections.Generic.IComparer(Of C1.Silverlight.FlexGrid.Row).Compare
            Dim v1 = x(_col)
            Dim v2 = y(_col)

            Dim cmp As Integer = 0
            If v1 Is Nothing AndAlso v2 IsNot Nothing Then
                cmp = -1
            ElseIf v1 IsNot Nothing AndAlso v2 Is Nothing Then
                cmp = +1
            ElseIf TypeOf v1 Is IComparable AndAlso TypeOf v2 Is IComparable Then
                cmp = DirectCast(v1, IComparable).CompareTo(v2)
            End If

            If _sortDirection = ListSortDirection.Descending Then
                cmp = -cmp
            End If

            Return cmp
        End Function
    End Class

End Class

Public Class SortCellFactory
    Inherits CellFactory
    Private _imgSrc As ImageSource

    Public Overrides Sub CreateColumnHeaderContent(grid As C1FlexGrid, bdr As Border, range As CellRange)

        Dim col = grid.Columns(range.Column)
        If col.Tag Is Nothing Then
            MyBase.CreateColumnHeaderContent(grid, bdr, range)

        ElseIf DirectCast(col.Tag, ListSortDirection) = ListSortDirection.Ascending Then
            Dim sp As New StackPanel()
            sp.Orientation = Orientation.Horizontal
            Dim tb As New TextBlock()
            If bdr.Child IsNot Nothing Then
                tb = TryCast(bdr.Child, TextBlock)
            End If
            bdr.Child = Nothing
            tb.Text = col.Header
            tb.VerticalAlignment = VerticalAlignment.Center
            sp.Children.Add(tb)
            _imgSrc = GetImageSource("SortUp.png")
            Dim img = New Image()
            img.Source = _imgSrc
            img.Width = 25
            img.Height = 15
            img.HorizontalAlignment = HorizontalAlignment.Center
            img.Stretch = Stretch.None
            sp.Children.Add(img)
            bdr.Child = sp

        ElseIf DirectCast(col.Tag, ListSortDirection) = ListSortDirection.Descending Then
            Dim sp As New StackPanel()
            sp.Orientation = Orientation.Horizontal
            Dim tb As New TextBlock()
            If bdr.Child IsNot Nothing Then
                tb = TryCast(bdr.Child, TextBlock)
            End If
            bdr.Child = Nothing
            tb.Text = col.Header
            tb.VerticalAlignment = VerticalAlignment.Center
            sp.Children.Add(tb)
            _imgSrc = GetImageSource("SortDn.png")
            Dim img = New Image()
            img.Source = _imgSrc
            img.Width = 25
            img.Height = 15
            img.HorizontalAlignment = HorizontalAlignment.Center
            img.Stretch = Stretch.None
            sp.Children.Add(img)
            bdr.Child = sp
        End If

    End Sub

    Public Shared Function GetImageSource(imageName As String) As ImageSource
        Dim bmp = New BitmapImage()
        bmp.CreateOptions = BitmapCreateOptions.None
        Dim fmt = "/C1FlexGrid_Sample;component/Resources/{0}"
        bmp.UriSource = New Uri(String.Format(fmt, imageName), UriKind.Relative)
        Return bmp
    End Function
End Class

